#include "editor.h"

#include <qworkspace.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qobjectlist.h>
#include <qvbox.h>
#include <qinputdialog.h>

static const char *filesave[] = {
"22 22 4 1",
"b c #c3c3c3",
"a c #808000",
"# c #000000",
"  c None",
"                      ",
"                      ",
"                      ",
"                      ",
"    ##############    ",
"    #a#        #b#    ",
"    #a#        ###    ",
"    #a#        #a#    ",
"    #a#        #a#    ",
"    #a#        #a#    ",
"    #a#        #a#    ",
"    #aa########aa#    ",
"    #aaaaaaaaaaaa#    ",
"    #aa#########a#    ",
"    #aa######bb#a#    ",
"    #aa######bb#a#    ",
"    #aa######bb#a#    ",
"     #############    ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *fileopen[] = {
"22 22 5 1",
"c c #c0c000",
"b c #808000",
"# c #000000",
"a c #ffffc0",
"  c None",
"                      ",
"                      ",
"                      ",
"                      ",
"            ###       ",
"           #   # #    ",
"                ##    ",
"    ###        ###    ",
"   #aaa#######        ",
"   #aaaaaaaaa#        ",
"   #aaaaaaaaa#        ",
"   #aaaa##########    ",
"   #aaa#bcbcbcbcb#    ",
"   #aa#bcbcbcbcb#     ",
"   #a#bcbcbcbcb#      ",
"   ##bcbcbcbcb#       ",
"   ###########        ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *fileprint[] = {
"22 22 6 1",
"b c #c6c6c6",
"# c #000000",
"c c #848484",
"d c #ffff00",
"  c None",
"a c #ffffff",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"        #########     ",
"       #aaaaaaaa#     ",
"       #a#####a#      ",
"      #aaaaaaaa#      ",
"      #a#####a####    ",
"     #aaaaaaaa#b#b#   ",
"    ##########b#b##   ",
"   #bbbbbbbbbb#b#b#   ",
"   #############bb#   ",
"   #bbbbbbcccbb#b#    ",
"   #bbbbbbdddbb###    ",
"   #############b#    ",
"    #bbbbbbbbb#b#     ",
"     ###########      ",
"                      ",
"                      ",
"                      "
};

static const char *editundo[] = {
"22 22 7 1",
"# c #808080",
"c c #000063",
"d c #a0a0a0",
"a c #000080",
"b c #0000c0",
"e c #0000ff",
"  c None",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"         #aaab#       ",
"     c daaaabbbb      ",
"     ccaad    bb#     ",
"     ccaa      bb     ",
"     cccc       e     ",
"     ccccc      e     ",
"                e     ",
"               de     ",
"              ded     ",
"              ed      ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *editredo[] = {
"22 22 7 1",
"# c #808080",
"d c #000063",
"c c #a0a0a0",
"b c #000080",
"a c #0000c0",
"e c #0000ff",
"  c None",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"       #abbb#         ",
"      aaaabbbbc d     ",
"     #aa    cbbdd     ",
"     aa      bbdd     ",
"     e       dddd     ",
"     e      ddddd     ",
"     e                ",
"     ec               ",
"     cec              ",
"      ce              ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *editcut[] = {
/* columns rows colors chars-per-pixel */
"22 22 3 1",
"a c #00009c",
"# c #000000",
"  c None",
"                      ",
"                      ",
"                      ",
"                      ",
"        #   #         ",
"        #   #         ",
"        #   #         ",
"        ## ##         ",
"         # #          ",
"         ###          ",
"          #           ",
"         a#a          ",
"         a aaa        ",
"       aaa a  a       ",
"      a  a a  a       ",
"      a  a a  a       ",
"      a  a  aa        ",
"       aa             ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *editcopy[] = {
"22 22 4 1",
"# c #000000",
"b c #0000c0",
"  c None",
"a c #ffffff",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"    ######            ",
"    #aaaa##           ",
"    #aaaa#a#          ",
"    #a##a#bbbbbb      ",
"    #aaaaabaaaabb     ",
"    #a####baaaabab    ",
"    #aaaaaba##abbbb   ",
"    #a####baaaaaaab   ",
"    #aaaaaba#####ab   ",
"    ######baaaaaaab   ",
"          ba#####ab   ",
"          baaaaaaab   ",
"          bbbbbbbbb   ",
"                      ",
"                      ",
"                      ",
"                      "
};

static const char *editpaste[] = {
"22 22 8 1",
"b c #9c9a00",
"d c #000063",
"# c #000000",
"e c #9c9a63",
"a c #ffff00",
"c c #9c9a9c",
"  c None",
"f c #ffffff",
"                      ",
"                      ",
"                      ",
"                      ",
"                      ",
"         ####         ",
"     #####aa#####     ",
"    #bcb#a##a#cbc#    ",
"    #cb#      #cb#    ",
"    #bc########cc#    ",
"    #cbcbcbcbcbcb#    ",
"    #bcbcbddddddd#    ",
"    #cbebcdfffffdd    ",
"    #bcbcbdfffffdfd   ",
"    #cbcbcdf###fdddd  ",
"    #bcbcbdffffffffd  ",
"    #cbcbcdf######fd  ",
"     #####dffffffffd  ",
"          dddddddddd  ",
"                      ",
"                      ",
"                      "
};

static const char* editfind[]={
"22 22 6 1",
"# c #000000",
"c c #ffffff",
"b c #dcdcdc",
"a c #a0a0a4",
"d c #dcdcdc",
"  c None",
"                      ",
"                      ",
"                      ",
"       ####           ",
"     a#bccd#a         ",
"     #ccaacc#a        ",
"    #dcaccccd#        ",
"    #cccccccc#        ",
"    #cccccccc#        ",
"    #dccccccd#        ",
"    a#cccccc#a        ",
"     a#dccd###        ",
"      a####a###       ",
"       aaaaaa###      ",
"            aa###     ",
"             aa###    ",
"              aa###   ",
"               aa#a   ",
"                aa    ",
"                      ",
"                      ",
"                      "};

/*
 *
 */

EditorMainWindow::EditorMainWindow( QWidget* parent, const char* name)
    : QMainWindow( parent, name )
{
    toolbar = new QToolBar( this, "operations" );
    addToolBar( toolbar, tr( "Operations" ), DockTop, TRUE );

    // file operations
  
    (void) new QToolButton( QPixmap( fileopen ), "Open File", QString::null,
			    this, SLOT(load()), toolbar, "open file" );

    (void) new QToolButton( QPixmap( filesave ), "Save File", QString::null,
			    this, SLOT(save()), toolbar, "save file" );

#ifndef QT_NO_PRINTER
    printer = new QPrinter;
    
    (void) new QToolButton( QPixmap( fileprint ), "Print File", QString::null,
			    this, SLOT(print()), toolbar, "print file" );
#endif

    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( "&File", file );

    file->insertItem( "&New", this, SLOT(newDoc()), CTRL+Key_N );
    file->insertItem( QPixmap( fileopen ), "&Open...", this, SLOT(load()), CTRL+Key_O );
    file->insertItem( QPixmap( filesave ), "&Save", this, SLOT(save()), CTRL+Key_S );
    file->insertItem( "Save &As...", this, SLOT(saveAs()) );
#ifndef QT_NO_PRINTER
    file->insertSeparator();
    file->insertItem( QPixmap( fileprint ), "&Print...", this, SLOT(print()), CTRL+Key_P );
#endif
    file->insertSeparator();
    file->insertItem( "&Close", this, SLOT(closeWindow()), CTRL+Key_W );

    // edit operations

    toolbar->addSeparator();

    (void) new QToolButton( QPixmap( editcut ), "Cut Edit", QString::null,
			    this, SLOT(cut()), toolbar, "cut edit" );

    (void) new QToolButton( QPixmap( editcopy ), "Copy Edit", QString::null,
			    this, SLOT(copy()), toolbar, "copy edit" );

    (void) new QToolButton( QPixmap( editpaste ), "Paste Edit", QString::null,
			    this, SLOT(paste()), toolbar, "paste edit" );

    toolbar->addSeparator();

    (void) new QToolButton( QPixmap( editundo ), "Undo Edit", QString::null,
			    this, SLOT(undo()), toolbar, "undo edit" );

    (void) new QToolButton( QPixmap( editredo ), "Redo Edit", QString::null,
			    this, SLOT(redo()), toolbar, "redo edit" );

    toolbar->addSeparator();
    
    (void)new QLabel( tr( " Find: " ), toolbar );
    findCombo = new QComboBox( TRUE, toolbar );
    findCombo->setAutoCompletion( TRUE );
    toolbar->setStretchableWidget( findCombo );
    (void) new QToolButton( QPixmap( editfind ), "Find String", QString::null,
			    this, SLOT( find() ), toolbar, "find string");
    
    QPopupMenu * edit = new QPopupMenu( this );
    menuBar()->insertItem( "&Edit", edit );

    edit->insertItem( QPixmap( editundo ), "&Undo", this, SLOT(undo()), CTRL+Key_Z);
    edit->insertItem( QPixmap( editredo ), "&Redo", this, SLOT(redo()), CTRL+Key_Y);
    edit->insertSeparator();
    edit->insertItem( QPixmap( editcut ), "&Cut", this, SLOT(cut()), CTRL+Key_X);
    edit->insertItem( QPixmap( editcopy ), "&C&opy", this, SLOT(copy()), CTRL+Key_C);
    edit->insertItem( QPixmap( editpaste ), "&Paste", this, SLOT(paste()), CTRL+Key_V);
    edit->insertSeparator();
    edit->insertItem( "&Goto Line...", this, SLOT(gotoLine()), CTRL+Key_G);
    edit->insertItem( QPixmap( editfind ), "&Find", this, SLOT(find()), CTRL+Key_F);

    // windows menu

    windowsMenu = new QPopupMenu( this );
    windowsMenu->setCheckable( TRUE );

    connect( windowsMenu, SIGNAL( aboutToShow() ),
	     this, SLOT( windowsMenuAboutToShow() ) );
    menuBar()->insertItem( "&Windows", windowsMenu );

    menuBar()->insertSeparator();
    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( "&Help", help );

    help->insertItem( "&About", this, SLOT(about()), Key_F1);
    help->insertItem( "About &Qt", this, SLOT(aboutQt()));

    QVBox* vb = new QVBox( this );
    vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    ws = new QWorkspace( vb );
    ws->setScrollBarsEnabled( TRUE );
    setCentralWidget( vb );

    statusBar()->message( "Ready", 2000 );
}


EditorMainWindow::~EditorMainWindow()
{
#ifndef QT_NO_PRINTER
    delete printer;
#endif
}

void EditorMainWindow::load( const QString& fn ) {
    if ( !fn.isEmpty() ) {
	EditorWindow* w = newDoc();
	w->load( fn );
    }  else {
	statusBar()->message( "Loading aborted", 2000 );
    }
} 

EditorWindow* EditorMainWindow::newDoc()
{
    EditorWindow* w = new EditorWindow( ws, 0, WDestructiveClose );
    connect( w, SIGNAL( message(const QString&, int) ), 
	     statusBar(), SLOT( message(const QString&, int )) );
    w->setCaption("unnamed document");
    // w->setIcon( QPixmap("document.xpm") );
    // show the very first window in maximized mode
    if ( ws->windowList().isEmpty() )
	w->showMaximized();
    else
	w->show();
    return w;
}

void EditorMainWindow::load()
{
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null, this );
    load ( fn );
}

void EditorMainWindow::save()
{
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->save();
}


void EditorMainWindow::saveAs()
{
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->saveAs();
}


void EditorMainWindow::print()
{
#ifndef QT_NO_PRINTER
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->print( printer );
#endif
}


void EditorMainWindow::closeWindow()
{
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->close();
}

void EditorMainWindow::undo() {
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->undo();
}

void EditorMainWindow::redo() {
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->redo();
}

void EditorMainWindow::cut() {
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->cut();
}

void EditorMainWindow::copy() {
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->copy();
}

void EditorMainWindow::paste() {
    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
	m->paste();
}

void EditorMainWindow::gotoLine() {
    bool ok = FALSE;
    int line = QInputDialog::getInteger("IF-2 Toolbox Editor", "Line number",
					0, 0, 100000, 1, &ok, this);
    if ( !ok )
        return;

    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
        m->gotoLine(line);
}

void EditorMainWindow::find() {
  QString needle = findCombo->currentText();
    if ( needle.isEmpty() )
        return;

    EditorWindow* m = (EditorWindow*)ws->activeWindow();
    if ( m )
        m->find( needle );
}

void EditorMainWindow::about()
{
    QMessageBox::about( this, "IF-2 Toolbox Editor",
			"IF-2 Toolbox Editor\nVerimag (C) All Rights Reserved.");
}


void EditorMainWindow::aboutQt()
{
    QMessageBox::aboutQt( this, "Qt Application" );
}


void EditorMainWindow::windowsMenuAboutToShow()
{
    windowsMenu->clear();
    int cascadeId = windowsMenu->insertItem("&Cascade", ws, SLOT(cascade() ) );
    int tileId = windowsMenu->insertItem("&Tile", ws, SLOT(tile() ) );
    int horTileId = windowsMenu->insertItem("Tile &Horizontally", this, SLOT(tileHorizontal() ) );
    if ( ws->windowList().isEmpty() ) {
	windowsMenu->setItemEnabled( cascadeId, FALSE );
	windowsMenu->setItemEnabled( tileId, FALSE );
	windowsMenu->setItemEnabled( horTileId, FALSE );
    }
    windowsMenu->insertSeparator();
    QWidgetList windows = ws->windowList();
    for ( int i = 0; i < int(windows.count()); ++i ) {
	int id = windowsMenu->insertItem(windows.at(i)->caption(),
					 this, SLOT( windowsMenuActivated( int ) ) );
	windowsMenu->setItemParameter( id, i );
	windowsMenu->setItemChecked( id, ws->activeWindow() == windows.at(i) );
    }
}

void EditorMainWindow::windowsMenuActivated( int id )
{
    QWidget* w = ws->windowList().at( id );
    if ( w )
	w->showNormal();
    w->setFocus();
}

void EditorMainWindow::tileHorizontal()
{
    // primitive horizontal tiling
    QWidgetList windows = ws->windowList();
    if ( !windows.count() )
	return;
    
    int heightForEach = ws->height() / windows.count();
    int y = 0;
    for ( int i = 0; i < int(windows.count()); ++i ) {
	QWidget *window = windows.at(i);
	if ( window->testWState( WState_Maximized ) ) {
	    // prevent flicker
	    window->hide();
	    window->showNormal();
	}
	int preferredHeight = window->minimumHeight()+window->parentWidget()->baseSize().height();
	int actHeight = QMAX(heightForEach, preferredHeight);
	
	window->parentWidget()->setGeometry( 0, y, ws->width(), actHeight );
	y += actHeight;
    }
}

/*
 *
 */

EditorWindow::EditorWindow( QWidget* parent, const char* name, WFlags wflags )
    : QMainWindow( parent, name, wflags )
{
    medit = new QTextEdit( this );
    medit->setFont( QFont("Fixed", 10, QFont::Normal) );
    medit->setWordWrap( QTextEdit::NoWrap );
    setFocusProxy( medit );
    setCentralWidget( medit );
}

EditorWindow::~EditorWindow()
{
}


void EditorWindow::load( const QString& fn )
{
    filename  = fn;
    QFile f( filename );
    if ( !f.open( IO_ReadOnly ) )
	return;

    QTextStream t(&f);
    QString s = t.read();
    medit->setText( s );
    f.close();
    
    setCaption( filename );
    emit message( QString("Loaded document %1").arg(filename), 2000 );
}

void EditorWindow::save()
{
    if ( filename.isEmpty() ) {
        saveAs();
        return;
    }

    QString text = medit->text();
    QFile f( filename );
    if ( !f.open( IO_WriteOnly ) ) {
        emit message( QString("Could not write to %1").arg(filename),
		      2000 );
        return;
    }

    QTextStream t( &f );
    t << text;
    f.close();

    setCaption( filename );

    emit message( QString( "File %1 saved" ).arg( filename ), 2000 );
}

void EditorWindow::saveAs()
{
    QString fn = QFileDialog::getSaveFileName( filename, QString::null, this );
    if ( !fn.isEmpty() ) {
        filename = fn;
        save();
    } else {
        emit message( "Saving aborted", 2000 );
    }
}

void EditorWindow::print( QPrinter* printer)
{
#ifndef QT_NO_PRINTER
    const int Margin = 10;
    int pageNo = 1;

    if ( printer->setup(this) ) {		// printer dialog
	emit message( "Printing...", 0 );
	QPainter p;
	if ( !p.begin( printer ) )
	    return;				// paint on printer
	p.setFont( medit->font() );
	int yPos        = 0;			// y position for each line
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( printer ); // need width/height
	// of printer surface
	for( int i = 0 ; i < medit->lines() ; i++ ) {
	    if ( Margin + yPos > metrics.height() - Margin ) {
		QString msg( "Printing (page " );
		msg += QString::number( ++pageNo );
		msg += ")...";
		emit message( msg, 0 );
		printer->newPage();		// no more room on this page
		yPos = 0;			// back to top of page
	    }
	    /* not yet implemented : textLine not available
	    p.drawText( Margin, Margin + yPos,
			metrics.width(), fm.lineSpacing(),
			ExpandTabs | DontClip,
			medit->textLine( i ) );
	    */
	    yPos = yPos + fm.lineSpacing();
	}
	p.end();				// send job to printer
	emit message( "Printing completed", 2000 );
    } else {
	emit message( "Printing aborted", 2000 );
    }
#endif
}

void EditorWindow::undo() {
  medit->undo();
}

void EditorWindow::redo() {
  medit->redo();
}

void EditorWindow::cut() {
  medit->cut();
}

void EditorWindow::copy() {
  medit->copy();
}

void EditorWindow::paste() {
  medit->paste();
}

void EditorWindow::gotoLine (const int line) {
  medit->setCursorPosition( line, 0 );
}

void EditorWindow::find( const QString& needle ) {
    int para = 0, index = 0;
    medit->getCursorPosition( &para, &index );
    if ( medit->find( needle, TRUE, FALSE, TRUE, &para, &index ) == TRUE )
	medit->setSelection( para, index, para, index + needle.length());
    else 
        QMessageBox::information(this, "IF-2 Toolbox Editor",
				 "The string you entered was not found", "Ok");
}
