
#ifndef EDITOR_H
#define EDITOR_H

#include <qmainwindow.h>
#include <qptrlist.h>
#include <qcombobox.h>

class QTextEdit;
class QToolBar;
class QPopupMenu;
class QWorkspace;
class QPopupMenu;
class QMovie;

class EditorWindow: public QMainWindow
{
    Q_OBJECT
public:
    EditorWindow( QWidget* parent = NULL, const char* name = NULL, WFlags f = 0 );
    ~EditorWindow();

    void load( const QString& fn );
    void save();
    void saveAs();
    void print( QPrinter* );

    void undo();
    void redo();
    void cut();
    void copy();
    void paste();
    void gotoLine( const int );
    void find( const QString& );

signals:
    void message(const QString&, int );

private:
    QTextEdit* medit;
    QString filename;
};


class EditorMainWindow: public QMainWindow
{
    Q_OBJECT
public:
    EditorMainWindow( QWidget* parent = NULL, const char* name = NULL );
    ~EditorMainWindow();

    void load( const QString& fn );

private slots:
    EditorWindow* newDoc();
    void load();
    void save();
    void saveAs();
    void print();
    void closeWindow();

    void undo();
    void redo();
    void cut();
    void copy();
    void paste();
    void gotoLine();
    void find();

    void tileHorizontal();

    void about();
    void aboutQt();

    void windowsMenuAboutToShow();
    void windowsMenuActivated( int id );

private:
    QPrinter *printer;
    QWorkspace* ws;
    QToolBar *toolbar;
    QComboBox* findCombo;
    QPopupMenu* windowsMenu;
};

#endif
