#ifndef FILEICONVIEW_H
#define FILEICONVIEW_H

#include <qiconset.h>
#include <qstring.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qtimer.h>
#include <qiconview.h>

class FileIconView;
class QDragObject;
class QResizeEvent;

/*****************************************************************************
 *
 * Class FileIconDrag
 *
 *****************************************************************************/

class FileIconDrag : public QIconDrag
{
    Q_OBJECT

public:
    FileIconDrag( QWidget * dragSource, const char* name = 0 );

    const char* format( int i ) const;
    QByteArray encodedData( const char* mime ) const;
    static bool canDecode( QMimeSource* e );
    void append( const QIconDragItem &item, const QRect &pr, const QRect &tr, const QString &url );

private:
    QStringList urls;

};

/*****************************************************************************
 *
 * Class FileIconView
 *
 *****************************************************************************/
class FileIconViewItem;
class FileIconView : public QIconView
{
    Q_OBJECT

public:
    FileIconView( const QString &dir, QWidget *parent = 0, const char *name = 0 );

    void setOpenItem( FileIconViewItem *i ) {
	openItem = i;
    }

public slots:
    void setDirectory( const QString &dir );
    void setDirectory( const QDir &dir );
    void newDirectory();
    QDir currentDir();

signals:
    void directoryChanged( const QString & );
    void startReadDir( int dirs );
    void readNextDir();
    void readDirDone();
    void enableUp( const bool );
    void enableMkdir( const bool );
    void runCommand( const QString & );

protected slots:
    void itemDoubleClicked( QIconViewItem *i );
    void slotDropped( QDropEvent *e, const QValueList<QIconDragItem> & );

    void viewBottom();
    void viewRight();
    void flowEast();
    void flowSouth();
    void itemTextTruncate();
    void itemTextWordWrap();
    void sortAscending();
    void sortDescending();
    void arrangeItemsInGrid() {
	QIconView::arrangeItemsInGrid( TRUE );
    }

    void mouseButton( int, QIconViewItem *item ); 
    void contextMenu( QIconViewItem *item ); 
    void openFolder();

public:
    void copy(QStrList& filelst, QString destination);
    void move(QStrList& filelst, QString destination);
protected:
    void rename( QIconViewItem *item );
    void remove( );

protected:
    void readDir( const QDir &dir );
    virtual QDragObject *dragObject();

    virtual void keyPressEvent( QKeyEvent *e );

    QDir viewDir;
    int newFolderNum;
    FileIconViewItem *openItem;

};

/*****************************************************************************
 *
 * Class FileIconViewItem
 *
 *****************************************************************************/

class FileIconViewItem : public QIconViewItem
{
    friend class FileIconView;

public:
    enum ItemType {
        File = 0,
        Dir,
        Link
    };

    FileIconViewItem( FileIconView *parent, QFileInfo *fi );
    virtual ~FileIconViewItem();

    ItemType type() const { return itemType; }
    QString filename() const { return itemFileName; }
    QFileInfo* fileinfo() const { return itemFileInfo; }

    virtual bool acceptDrop( const QMimeSource *e ) const;

    virtual void setText( const QString &text );
    virtual QPixmap *pixmap() const;

    virtual void dragEntered();
    virtual void dragLeft();

    void paintItem( QPainter *p, const QColorGroup &cg );

protected:
    virtual void dropped( QDropEvent *e, const QValueList<QIconDragItem> & );

    ItemType itemType;
    QString itemFileName;
    QFileInfo *itemFileInfo;
    bool checkSetText;
    QTimer timer;

};


#endif
