
#include "toolbox.h"
#include "toolsetup.h"

#include "fileiconview.h"

#include <qsplitter.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qmenubar.h>

Toolbox::Toolbox() 
  : QMainWindow( 0, "toolbox" ), openFileInfo(0), ifPath(getenv("IF"))
{
  QSplitter * splitter = new QSplitter( this );
  
  fileBrowser = new FileBrowser( splitter );
  fileBrowser->fileView()->setDirectory( QDir::current().path() );

  taskManager = new TaskManager( splitter );

  setCentralWidget( splitter );

  connect( fileBrowser->fileView(), SIGNAL( contextMenuRequested( QIconViewItem*, const QPoint& ) ),
	   this, SLOT( contextMenu( QIconViewItem*, const QPoint&  ) ) );
  connect( fileBrowser->fileView(), SIGNAL( directoryChanged( const QString& ) ),
	   taskManager, SLOT( directoryChanged( const QString&  ) ) );


  QPopupMenu* filemenu = new QPopupMenu( this, "File");
  menuBar()->insertItem("&File", filemenu);

  filemenu->insertItem( "&Quit", this, SLOT( close() ), CTRL+Key_Q );

  editor = new EditorMainWindow ( this, "editor" );
  editor->setCaption( "IF-2 Toolbox Editor" );
}

void Toolbox::contextMenu( QIconViewItem* item, const QPoint& ) 
{ 
  if ( item == NULL )
    return;
  
  openFileInfo = ((FileIconViewItem*) item)->fileinfo();

  if ( !openFileInfo->isFile() || 
       !openFileInfo->isReadable() )
    return;
  
  QString extension = openFileInfo->extension( FALSE );

  QPopupMenu menu ( this );
  
  menu.insertItem( "&Open...", this, SLOT( open() ) );
  
  if (extension == "if") {
    menu.insertSeparator();
    menu.insertItem("&Compile", this, SLOT( compile() ) );
    menu.insertItem("&Generate...", this, SLOT( generate() ) );
    menu.insertItem("&Optimize...", this, SLOT( optimize() ) );
    menu.insertSeparator();
    menu.insertItem("&Lash...", this, SLOT( lash() ) );
    menu.insertItem("&Trex...", this, SLOT( trex() ) );
  }
  
  else if (extension == "aut" ) {
    menu.insertSeparator();
    menu.insertItem("&Minimize...", this, SLOT( minimize() ) );
    menu.insertItem("&Visualize", this, SLOT( visualize() ) );
  }

  
  menu.setMouseTracking( TRUE );
  menu.exec( QCursor::pos() );
  
  openFileInfo = 0;
}

void Toolbox::open() {
  QString filepath( openFileInfo->filePath() );

  editor->show();
  editor->load( filepath );
}

void Toolbox::compile() {
  QString filename( openFileInfo->fileName() );
  filename.replace (filename.length() - 2, 2, "x");

  QString task = "make -f " + 
    ifPath + "/examples/Makefile " +
    filename;

  taskManager->runTask( task );
}

void Toolbox::generate() {
  QString filename ( openFileInfo->fileName() );

  GeneratorSetup dlg( this, filename );
  dlg.setup();

  if (dlg.exec() == QDialog::Accepted) 
    taskManager->runTask( dlg.command() );
}

void Toolbox::optimize() {
  QString filename ( openFileInfo->fileName() );

  OptimizerSetup dlg( this, filename );
  dlg.setup();

  if (dlg.exec() == QDialog::Accepted)
    taskManager->runTask( dlg.command() );
}

void Toolbox::lash() {
  QString filename ( openFileInfo->fileName() );

  LashSetup dlg( this, filename );
  dlg.setup();

  if (dlg.exec() == QDialog::Accepted)
    taskManager->runTask( dlg.command() );
}

void Toolbox::trex() {
  QString filename ( openFileInfo->fileName() );

  TrexSetup dlg( this, filename );
  dlg.setup();

  if (dlg.exec() == QDialog::Accepted)
    taskManager->runTask( dlg.command() );
}

void Toolbox::minimize() {
  QString filename ( openFileInfo->fileName() );

  AldebaranSetup dlg( this, filename );
  dlg.setup();

  if (dlg.exec() == QDialog::Accepted)
    taskManager->runTask( dlg.command() );
}

void Toolbox::visualize() {
  QString filename ( openFileInfo->fileName() );
  
  QString task = "aut2eps " + filename;

  taskManager->runTask( task );
}
